import os
import numpy as np
from PIL import Image

import paddle.fluid as fluid
import time


def create_config(model_path, mode='fluid', batch_size=1, min_subgraph_size=3):
    model_file = os.path.join(model_path, '__model__')
    params_file = os.path.join(model_path, '__params__')
    config = fluid.core.AnalysisConfig(model_file, params_file)
    config.disable_gpu()
    config.enable_mkldnn()
    if mode == 'trt_int8':
        config.enable_tensorrt_engine(
            workspace_size=1 << 30,
            max_batch_size=batch_size,
            min_subgraph_size=min_subgraph_size,
            precision_mode=fluid.core.AnalysisConfig.Precision.Int8,
            use_static=False,
            use_calib_mode=True)
        print('Run inference by TRT INT8.')
    elif mode == 'trt_fp32':
        config.enable_tensorrt_engine(
            workspace_size=1 << 30,
            max_batch_size=batch_size,
            min_subgraph_size=min_subgraph_size,
            precision_mode=fluid.core.AnalysisConfig.Precision.Float32,
            use_static=False)
        print('Run inference by TRT FP32.')
    elif mode == 'fluid':
        print('Run inference by Fluid FP32.')
    else:
        print('Wrong mode, only support trt_int8, trt_fp32, fluid.')
    return config


def create_tensor(np_data, dtype):
    dtype_map = {
        'float32': fluid.core.PaddleDType.FLOAT32,
        'int64': fluid.core.PaddleDType.INT64,
        'int32': fluid.core.PaddleDType.INT32
    }

    t = fluid.core.PaddleTensor()
    t.dtype = dtype_map[dtype]
    t.shape = np_data.shape
    buf = np_data.flatten().tolist()
    t.data = fluid.core.PaddleBuf(buf)
    return t


image_path = './0--Parade/0_Parade_Parade_0_102.jpg'
image = Image.open(image_path).convert('RGB')
data = np.array(image.resize((640, 640), Image.ANTIALIAS)).transpose(
    [2, 0, 1])[np.newaxis, :, :]

model_path='./output/blazeface/model_old/blazeface_nas'
config = create_config(model_path, mode='fluid', min_subgraph_size=3)

predict = fluid.core.create_paddle_predictor(config)
inputs = [create_tensor(data, 'float32')]
for i in range(10):
    outs = predict.run(inputs)

t1 = time.time()
for i in range(100):
    outs = predict.run(inputs)
t2 = time.time()
ms = (t2 - t1) * 1000.0 / float(100)
print("Inference: {} ms per image".format(ms))